@@ -4,17 +4,18 @@ import atexit
 import functools
 import inspect
 import os
+import sys
+import warnings
 import weakref
 
 from .actions import Action
 from .actions import CallPrinter
 from .actions import CodePrinter
 from .actions import Debugger
+from .actions import ErrorSnooper
 from .actions import Manhole
 from .actions import VarsPrinter
 from .actions import VarsSnooper
-from .config import THREADING_SUPPORT_ALIASES
-from .config import load_config
 
 try:
     if os.environ.get("PUREPYTHONHUNTER"):
@@ -38,17 +39,17 @@ except ImportError:
     from .predicates import Query
     from .tracer import Tracer
 
-
 try:
     from ._version import version as __version__
 except ImportError:
-    __version__ = '3.0.4'
+    __version__ = '3.1.3'
 
 __all__ = (
     'And',
     'CallPrinter',
     'CodePrinter',
     'Debugger',
+    'ErrorSnooper',
     'From',
     'Manhole',
     'Not',
@@ -62,8 +63,60 @@ __all__ = (
     'stop',
     'trace',
 )
+THREADING_SUPPORT_ALIASES = (
+    "threading_support", "threads_support", "thread_support",
+    "threadingsupport", "threadssupport", "threadsupport",
+    "threading", "threads", "thread",
+)
 _last_tracer = None
+_default_trace_args = None
 _default_config = {}
+_default_stream = sys.stderr
+
+
+def _prepare_config(*args, **kwargs):
+    _default_config.clear()
+    _default_config.update((key.lower(), val) for key, val in kwargs.items())
+    options = {}
+    predicates = []
+
+    for key, value in list(_default_config.items()):
+        if key in THREADING_SUPPORT_ALIASES or key == "clear_env_var":
+            options[key] = _default_config.pop(key)
+            continue
+        elif key in (
+            # builtin actions config
+            "klass",
+            "stream",
+            "force_colors",
+            "force_pid",
+            "filename_alignment",
+            "thread_alignment",
+            "pid_alignment",
+            "repr_limit",
+            "repr_func",
+        ):
+            continue
+
+        try:
+            Q(**{key: value})
+        except TypeError:
+            pass
+        else:
+            options[key] = _default_config.pop(key)
+            continue
+
+        _default_config.pop(key)
+        sys.stderr.write("Discarded config from PYTHONHUNTERCONFIG {}={!r}: Unknown option\n".format(
+            key, value))
+    for position, predicate in enumerate(args):
+        if callable(predicate):
+            predicates.append(predicate)
+        else:
+            sys.stderr.write("Discarded config from PYTHONHUNTERCONFIG {} (position {}): Not a callable\n".format(
+                predicate, position))
+
+    return predicates, options
 
 
 def Q(*predicates, **query):
@@ -71,6 +124,9 @@ def Q(*predicates, **query):
     Helper that handles situations where :class:`hunter.predicates.Query` objects (or other callables)
     are passed in as positional arguments - it conveniently converts those to a
     :class:`hunter.predicates.And` predicate.
+
+    See Also:
+         :class:`hunter.predicates.Query`
     """
     optional_actions = query.pop("actions", [])
     if "action" in query:
@@ -136,6 +192,8 @@ def And(*predicates, **kwargs):
 
     Returns: A :class:`hunter.predicates.And` instance.
 
+    See Also:
+         :class:`hunter.predicates.And`
     """
     if kwargs:
         predicates += Query(**kwargs),
@@ -153,9 +211,11 @@ def Or(*predicates, **kwargs):
 
     Returns: A :class:`hunter.predicates.Or` instance.
 
+    See Also:
+         :class:`hunter.predicates.Or`
     """
     if kwargs:
-        predicates += tuple(Query(**{k: v}) for k, v in kwargs.items())
+        predicates += tuple(Query(**{key: value}) for key, value in kwargs.items())
     return _flatten(_Or, *predicates)
 
 
@@ -169,6 +229,9 @@ def Not(*predicates, **kwargs):
         **kwargs: Arguments that may be passed to :class:`hunter.predicates.Query`.
 
     Returns: A :class:`hunter.predicates.Not` instance (possibly containing a :class:`hunter.predicates.And` instance).
+
+    See Also:
+         :class:`hunter.predicates.Not`
     """
     if kwargs:
         predicates += Query(**kwargs),
@@ -178,24 +241,34 @@ def Not(*predicates, **kwargs):
         return _Not(*predicates)
 
 
-def From(predicate=None, condition=None, watermark=0, **kwargs):
+def From(condition=None, predicate=None, watermark=0, **kwargs):
     """
-    Helper that converts keyword arguments to a ``From(Q(**kwargs))``.
+    Helper that converts keyword arguments to ``From(condition=Q(**normal_kwargs), predicate=Q(**rel_kwargs)``
+    where ``rel_kwargs`` are all the kwargs that start with "depth" or "calls".
 
     Args:
         condition (callable): A callable that returns True/False or a :class:`hunter.predicates.Query` object.
         predicate (callable): Optional callable that returns True/False or a :class:`hunter.predicates.Query` object to
             run after ``condition`` first returns ``True``.
-        watermark (int): Depth difference to switch off and wait again on ``condition``.
-        **kwargs: Arguments that are passed to :func:`hunter.Q`
+        **kwargs: Arguments that are passed to :func:`hunter.Q`. Any kwarg that starts with "depth" or "calls" will be included `predicate`.
+
+    Examples:
+        ``From(function='foobar', depth_lt=5)`` coverts to ``From(Q(function='foobar'), Q(depth_lt=5))``.
+        The depth filter is moved in the predicate because it would not have any effect as a condition - it stop being called after it
+        returns True, thus it doesn't have the intended effect (a limit to how deep to trace from ``foobar``).
+
+    See Also:
+         :class:`hunter.predicates.From`
     """
-    if predicate is None and condition is None and watermark == 0:
-        return _From(Q(**kwargs))
+    if predicate is None and condition is None:
+        condition_kwargs = {key: value for key, value in kwargs.items() if not key.startswith('depth') and not key.startswith('calls')}
+        predicate_kwargs = {key: value for key, value in kwargs.items() if key.startswith('depth') or key.startswith('calls')}
+        return _From(Q(**condition_kwargs), Q(**predicate_kwargs), watermark)
     else:
         if kwargs:
             raise TypeError("Unexpected arguments {}. Don't combine positional with keyword arguments.".format(
                 kwargs.keys()))
-        return _From(predicate, condition, watermark)
+        return _From(condition, predicate, watermark)
 
 
 def stop():
@@ -204,7 +277,9 @@ def stop():
     """
     global _last_tracer
 
-    if _last_tracer is not None:
+    if _last_tracer is None:
+        warnings.warn('There is no tracer to stop.')
+    else:
         _last_tracer.stop()
         _last_tracer = None
 
@@ -221,6 +296,14 @@ def _prepare_predicate(*predicates, **options):
     return Q(*predicates, **options)
 
 
+def _apply_config(predicates, options):
+    if _default_trace_args is None:
+        return predicates, options
+    else:
+        config_predicates, config_options = _default_trace_args
+        return predicates + tuple(config_predicates), dict(config_options, **options)
+
+
 def trace(*predicates, **options):
     """
     Starts tracing. Can be used as a context manager (with slightly incorrect semantics - it starts tracing
@@ -244,10 +327,13 @@ def trace(*predicates, **options):
         action: Action to run if all the predicates return ``True``. Default: ``CodePrinter``.
         actions: Actions to run (in case you want more than 1).
         **kwargs: for convenience you can also pass anything that you'd pass to :obj:`hunter.Q`
+
+    See Also:
+         :class:`hunter.tracer.Tracer` or :class:`hunter.event.Event`
     """
     global _last_tracer
 
-    predicates, options = load_config(predicates, options)
+    predicates, options = _apply_config(predicates, options)
 
     clear_env_var = options.pop("clear_env_var", False)
     threading_support = None
@@ -315,8 +401,26 @@ def wrap(function_to_trace=None, **trace_options):
                 return func(*args, **kwargs)
             finally:
                 local_tracer.stop()
+
         return tracing_wrapper
+
     if function_to_trace is None:
         return tracing_decorator
     else:
         return tracing_decorator(function_to_trace)
+
+
+def load_config(*args, **kwargs):
+    global _default_trace_args
+    try:
+        if args or kwargs:
+            _default_trace_args = _prepare_config(*args, **kwargs)
+        else:
+            _default_trace_args = eval("_prepare_config({})".format(os.environ.get("PYTHONHUNTERCONFIG", '')))
+    except Exception as exc:
+        sys.stderr.write("Failed to load hunter config from PYTHONHUNTERCONFIG {[PYTHONHUNTERCONFIG]!r}: {!r}\n".format(
+            os.environ, exc))
+        _default_trace_args = (), ()
+
+
+load_config()
